from tests.conftest import PROTOCOL, load_test_data_file, IaRequestsMock, ia_call

try:
    import ujson as json
except ImportError:
    import json

import responses


def test_ia_search_itemlist(capsys):
    test_scrape_response = load_test_data_file('scrape_response.json')

    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        url1 = ('{0}//archive.org/services/search/v1/scrape'
                '?q=collection%3Aattentionkmartshoppers'
                '&REQUIRE_AUTH=true&count=10000'.format(PROTOCOL))
        url2 = ('{0}//archive.org/services/search/v1/scrape?'
                'cursor=W3siaWRlbnRpZmllciI6IjE5NjEtTC0wNTkxNCJ9XQ%3D%3D'
                '&REQUIRE_AUTH=true&q=collection%3Aattentionkmartshoppers'
                '&count=10000'.format(PROTOCOL))
        rsps.add(responses.POST, url1,
                 body=test_scrape_response,
                 match_querystring=True)
        _j = json.loads(test_scrape_response)
        del _j['cursor']
        _r = json.dumps(_j)
        rsps.add(responses.POST, url2,
                 body=_r,
                 match_querystring=True)
        ia_call(['ia', 'search', 'collection:attentionkmartshoppers', '--itemlist'])

    out, err = capsys.readouterr()
    assert len(out.split()) == 200


def test_ia_search_num_found(capsys):
    with IaRequestsMock(assert_all_requests_are_fired=False) as rsps:
        url = ('{0}//archive.org/services/search/v1/scrape'
               '?q=collection%3Anasa&total_only=true'
               '&REQUIRE_AUTH=true&count=10000'.format(PROTOCOL))
        rsps.add(responses.POST, url,
                 body='{"items":[],"count":0,"total":50}',
                 match_querystring=True)

        ia_call(['ia', 'search', 'collection:nasa', '--num-found'])
    out, err = capsys.readouterr()
    assert out == '50\n'



# -*- coding: utf-8 -*-
#
# The internetarchive module is a Python/CLI interface to Archive.org.
#
# Copyright (C) 2012-2016 Internet Archive
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Dark an item.

usage:
    ia dark <identifier>... --comment=<comment>
    ia dark --help

options:
    -h, --help
    -c, --comment=<comment>   Curation comment.
"""
from __future__ import print_function, absolute_import
import os
import sys

import six
from docopt import docopt, printable_usage
from schema import Schema, Use, Or, And, SchemaError

from internetarchive.cli.argparser import get_args_dict


def main(argv, session):
    args = docopt(__doc__, argv=argv)

    # Validate args.
    s = Schema({
        str: Use(bool),
        '--comment': str,
        '<identifier>': list,
    })

    # Filenames should be unicode literals. Support PY2 and PY3.
    if six.PY2:
        args['<file>'] = [f.decode('utf-8') for f in args['<file>']]

    try:
        args = s.validate(args)
    except SchemaError as exc:
        sys.stderr.write('{0}\n{1}\n'.format(
            str(exc), printable_usage(__doc__)))
        sys.exit(1)

    errors = False
    if args['<identifier>'] == ['-']:
        args['<identifier>'] = sys.stdin
    for identifier in args['<identifier>']:
        item = session.get_item(identifier)
        if not item.exists:
            print('warning: item does not exist - {}'.format(item.identifier),
                  file=sys.stderr)
        r = item.dark(args['--comment'])
        if r is None:
            print('warning: no task submitted, item already dark - {}'.format(item.identifier),
                  file=sys.stderr)
        elif r.status_code != 200:
            print('error: task failed, item not dark - {}'.format(item.identifier),
                  file=sys.stderr)
            errors = True
        else:
            print('success: item darked - {}'.format(item.identifier))

    if errors:
        print('\n*** Not all items were darked! ***\n')
        sys.exit(1)



# -*- coding: utf-8 -*-
#
# The internetarchive module is a Python/CLI interface to Archive.org.
#
# Copyright (C) 2012-2019 Internet Archive
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Configure 'ia' with your Archive.org credentials.

usage:
    ia configure
    ia configure --username=<username> --password=<password>
    ia configure --netrc
    ia configure [--help]

options:
    -h, --help
    -u, --username=<username>  Provide username as an option rather than
                               providing it interactively.
    -p, --password=<password>  Provide password as an option rather than
                               providing it interactively.
    -n, --netrc                Use netrc file for login.
"""
from __future__ import absolute_import, print_function, unicode_literals
import sys
import netrc

from docopt import docopt

from internetarchive import configure
from internetarchive.exceptions import AuthenticationError


def main(argv, session):
    args = docopt(__doc__, argv=argv)
    try:
        # CLI params.
        if args['--username'] and args['--password']:
            config_file_path = configure(args['--username'],
                                         args['--password'],
                                         session.config_file)
            print('Config saved to: {0}'.format(config_file_path))

        # Netrc
        elif args['--netrc']:
            print("Configuring 'ia' with netrc file...")
            try:
                n = netrc.netrc()
            except netrc.NetrcParseError as exc:
                print('error: netrc.netrc() cannot parse your .netrc file.')
                sys.exit(1)
            username, _, password = n.hosts['archive.org']
            config_file_path = configure(username, password,
                                         config_file=session.config_file)
            print('Config saved to: {0}'.format(config_file_path))

        # Interactive input.
        else:
            print("Enter your Archive.org credentials below to configure 'ia'.\n")
            config_file_path = configure(config_file=session.config_file)
            print('\nConfig saved to: {0}'.format(config_file_path))

    except AuthenticationError as exc:
        print('\nerror: {0}'.format(str(exc)))
        sys.exit(1)



import sys
import os
import time

from tests.conftest import call_cmd, NASA_EXPECTED_FILES, files_downloaded


def test_no_args(tmpdir_ch):
    call_cmd('ia --insecure download nasa')
    assert files_downloaded(path='nasa') == NASA_EXPECTED_FILES


def test_https(tmpdir_ch):
    if sys.version_info < (2, 7, 9):
        stdout, stderr = call_cmd('ia download nasa', expected_exit_code=1)
        assert 'You are attempting to make an HTTPS' in stderr
    else:
        call_cmd('ia download nasa')
        assert files_downloaded(path='nasa') == NASA_EXPECTED_FILES


def test_dry_run():
    nasa_url = 'http://archive.org/download/nasa/'
    expected_urls = set([nasa_url + f for f in NASA_EXPECTED_FILES])

    stdout, stderr = call_cmd('ia --insecure download --dry-run nasa')
    output_lines = stdout.split('\n')
    dry_run_urls = set([x.strip() for x in output_lines if x and 'nasa:' not in x])

    assert expected_urls == dry_run_urls


def test_glob(tmpdir_ch):
    expected_files = set([
        'globe_west_540.jpg',
        'NASAarchiveLogo.jpg',
        'globe_west_540_thumb.jpg',
        '__ia_thumb.jpg',
    ])

    call_cmd('ia --insecure download --glob="*jpg" nasa')
    assert files_downloaded(path='nasa') == expected_files


def test_format(tmpdir_ch):
    call_cmd('ia --insecure download --format="Archive BitTorrent" nasa')
    assert files_downloaded(path='nasa') == set(['nasa_archive.torrent'])


def test_clobber(tmpdir_ch):
    cmd = 'ia --insecure download nasa nasa_meta.xml'
    call_cmd(cmd)
    assert files_downloaded('nasa') == set(['nasa_meta.xml'])

    stdout, stderr = call_cmd(cmd)
    assert files_downloaded('nasa') == set(['nasa_meta.xml'])
    assert 'nasa: . - success' == stdout


def test_checksum(tmpdir_ch):
    call_cmd('ia --insecure download nasa nasa_meta.xml')
    assert files_downloaded('nasa') == set(['nasa_meta.xml'])

    stdout, stderr = call_cmd('ia --insecure download --checksum nasa nasa_meta.xml')
    assert files_downloaded('nasa') == set(['nasa_meta.xml'])

    assert 'nasa: . - success' == stdout


def test_no_directories(tmpdir_ch):
    call_cmd('ia --insecure download --no-directories nasa nasa_meta.xml')
    assert files_downloaded('.') == set(['nasa_meta.xml'])


def test_destdir(tmpdir_ch):
    cmd = 'ia --insecure download --destdir=thisdirdoesnotexist/ nasa nasa_meta.xml'
    stdout, stderr = call_cmd(cmd, expected_exit_code=1)

    assert '--destdir must be a valid path to a directory.' in stderr

    tmpdir_ch.mkdir('thisdirdoesnotexist/')
    call_cmd(cmd)
    assert files_downloaded('thisdirdoesnotexist/nasa') == set(['nasa_meta.xml'])

    tmpdir_ch.mkdir('dir2/')
    cmd = ('ia --insecure download --no-directories --destdir=dir2/ '
           'nasa nasa_meta.xml')
    call_cmd(cmd)
    assert files_downloaded('dir2') == set(['nasa_meta.xml'])


def test_no_change_timestamp(tmpdir_ch):
    # TODO: Handle the case of daylight savings time

    now = time.time()
    call_cmd('ia --insecure download --no-change-timestamp nasa')

    for path, dirnames, filenames in os.walk(str(tmpdir_ch)):
        for d in dirnames:
            p = os.path.join(path, d)
            assert os.stat(p).st_mtime >= now

        for f in filenames:
            p = os.path.join(path, f)
            assert os.stat(p).st_mtime >= now



def make_url(protocol, host=None, path=None):
    setattr(self, path, url_format.format(self._itm_obj, path=path))
    self._paths.append(path)


#class URLs:
#    def __init__(self, itm_obj):
#        self._itm_obj = itm_obj
#        self._paths = []
#        self._make_URL('details')
#        self._make_URL('metadata')
#        self._make_URL('download')
#        self._make_URL('history')
#        self._make_URL('edit')
#        self._make_URL('editxml')
#        self._make_URL('manage')
#        if self._itm_obj.metadata.get('mediatype') == 'collection':
#            self._make_tab_URL('about')
#            self._make_tab_URL('collection')
#
#    def _make_tab_URL(self, tab):
#        """Make URLs for the separate tabs of Collections details page."""
#        self._make_URL(tab, self.details + "&tab={tab}".format(tab=tab))
#
#    DEFAULT_URL_FORMAT = '{0.session.protocol}//archive.org/{path}/{0.identifier}'
#
#    def _make_URL(self, path, url_format=DEFAULT_URL_FORMAT):
#        setattr(self, path, url_format.format(self._itm_obj, path=path))
#        self._paths.append(path)
#
#    def __str__(self):
#        return "URLs ({1}) for {0.identifier}" \
#               .format(self._itm_obj, ', '.join(self._paths))
#
